from scipy import *
from numpy import *
import matplotlib.pyplot as plt
plt.rcParams['mathtext.fontset'] = 'cm'

#File Read
DataFolder = "./AtomPos_UnitCell"
Atom_Pos_File    = DataFolder+"/AtomPos_Fig10.xyz" #Atomic Positions
Lattice_Vec_File = DataFolder+"/Lattice_Fig10.csv" #Unit Cell Lattice Vector

#Parameter Set
t1      = -1.0  #Hopping Parameter
C_C_len =  1.42 #C-C lentgh

#Atomic Positions Read
Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
Atom_Pos       = zeros((len(Atom_Pos_read), 2))
for i in range(len(Atom_Pos)):
    Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])

#Unit Cell Read
Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
a1 = Lattice_Vec[0]
a2 = Lattice_Vec[1]
a3 = Lattice_Vec[2]

#Reciprocal lattice vector calc. (2D)
b1=((2*pi)*cross(a2, a3)/(dot(a1, cross(a2, a3))))[0:2]
b2=((2*pi)*cross(a3, a1)/(dot(a2, cross(a3, a1))))[0:2]
b3=((2*pi)*cross(a1, a2)/(dot(a3, cross(a1, a2))))[0:2]
a1 = Lattice_Vec[0][0:2] #2D vec.
a2 = Lattice_Vec[1][0:2]
a3 = Lattice_Vec[2][0:2]

#Hopping Matrix Generation
def Ham(k_vec, a_1, a_2):
    H_1st = zeros((len(Atom_Pos), len(Atom_Pos)), dtype=complex64)
    for i in range(len(Atom_Pos)):
        for j in range(len(Atom_Pos)):
            if abs(linalg.norm(Atom_Pos[i]-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1
            if abs(linalg.norm((Atom_Pos[i]+a_1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_1)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm((Atom_Pos[i]+a_2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_2)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_2)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_2)))
    return H_1st


#Band Plot for Panel (c)
band_slice = 60

#K-Point
K = (2/3)*(b1 + (b2/2))
#M-Point
M = (1/2)*(b1 + b2)

#Eigenvalues Calc. Function
def EigVal_k_all(k_vec):
    eig_all=linalg.eigh(Ham(k_vec, a1, a2))[0]
    return eig_all

#Band Calc.
Band_K_points = []
#G→K
for i in range(int(2*band_slice)):
    Band_K_points.append(i*K/int(2*band_slice))
#K→M
for i in range(int(1*band_slice)):
    Band_K_points.append((int(1*band_slice)-i)*K/int(1*band_slice)+(i)*M/int(1*band_slice))
#M→G
for i in range(int(sqrt(3)*band_slice)+1):
    Band_K_points.append((int(sqrt(3)*band_slice)-i)*M/int(sqrt(3)*band_slice))

#2-Site Model
tau1 = 6/sqrt(4843)
tau2 = tau1
tau3 = tau1

#Hamiltonian for 2-Site Model
def Ham_2atom(k_vec, a_1, a_2, tau1, tau2, tau3):
    H_0      = zeros((2, 2), dtype=complex64)
    H_1      = zeros((2, 2), dtype=complex64)
    H_2      = zeros((2, 2), dtype=complex64)
    H_0[0][1]= tau1
    H_0[1][0]= tau1
    H_1[0][1]= tau2*exp(-1.0j*(k_vec.dot(a_1)))
    H_1[1][0]= tau2*exp( 1.0j*(k_vec.dot(a_1)))
    H_2[0][1]= tau3*exp(-1.0j*(k_vec.dot(a_2)))
    H_2[1][0]= tau3*exp( 1.0j*(k_vec.dot(a_2)))
    H        = H_0 + H_1 + H_2
    return H 

#Eigenvalues Calc. Function for 2-Site Model
def EigVal_k_2atom(k_vec, a1, a2, tau1, tau2, tau3):
    eig_n=linalg.eigh(Ham_2atom(k_vec, a1, a2, tau1, tau2, tau3))[0][1]
    return eig_n
    
#Original Model Band
EigenEnergy_Band_k = []
for k in range(len(Band_K_points)):    
    EigenEnergy_Band_k.append(EigVal_k_all(Band_K_points[k]))
EigenEnergy_Band = array(EigenEnergy_Band_k).T

#2-site Model Band
EigenEnergy_Eff_k = []
for k in range(len(Band_K_points)):
    EigenEnergy_Eff_k.append(EigVal_k_2atom(Band_K_points[k], a1, a2, tau1, tau2, tau3))
EigenEnergy_Eff = array(EigenEnergy_Eff_k).T

#Plot Figure
plt.figure(figsize=(6,4), dpi=300)
plt.ylabel("Energy in $t$")
plt.xlim(0, len(Band_K_points)-1)
plt.ylim(-1.0, 1.0)
for i in range(len(Atom_Pos)):
    plt.plot(EigenEnergy_Band[i], "k-")
plt.plot( EigenEnergy_Eff, "r--")
plt.plot(-EigenEnergy_Eff, "r--")
plt.xticks([0, int(2*band_slice), int(3*band_slice), len(Band_K_points)-1], [r"$\mathrm{\Gamma}$", "$\mathrm{K}$", "$\mathrm{M}$", r"$\mathrm{\Gamma}$"])
plt.axvline(int(2*band_slice), c='k', ls='--')
plt.axvline(int(3*band_slice), c='k', ls='--')
plt.grid()

#Show Figure
plt.show()