from scipy import *
from numpy import *
import matplotlib.pyplot as plt
plt.rcParams['mathtext.fontset'] = 'cm'

#File Read
DataFolder = "./AtomPos_UnitCell"
Atom_Pos_File    = DataFolder+"/AtomPos_Fig5_A1cd.xyz" #Atomic Positions
Lattice_Vec_File = DataFolder+"/Lattice_Fig5_A1cd.csv" #Unit Cell Lattice Vector

#Parameter Set
t1      = -1.0  #Hopping Parameter
C_C_len =  1.42 #C-C lentgh

#Atomic Positions Read
Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
Atom_Pos       = zeros((len(Atom_Pos_read), 2))
for i in range(len(Atom_Pos)):
    Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])

#Unit Cell Read
Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
a1 = Lattice_Vec[0]
a2 = Lattice_Vec[1]
a3 = Lattice_Vec[2]

#Reciprocal lattice vector calc. (2D)
b1=((2*pi)*cross(a2, a3)/(dot(a1, cross(a2, a3))))[0:2]
b2=((2*pi)*cross(a3, a1)/(dot(a2, cross(a3, a1))))[0:2]
b3=((2*pi)*cross(a1, a2)/(dot(a3, cross(a1, a2))))[0:2]
a1 = Lattice_Vec[0][0:2] #2D vec.
a2 = Lattice_Vec[1][0:2]
a3 = Lattice_Vec[2][0:2]

#Hopping Matrix Generation
def Ham(k_vec, a_1, a_2):
    H_1st = zeros((len(Atom_Pos), len(Atom_Pos)), dtype=complex64)
    for i in range(len(Atom_Pos)):
        for j in range(len(Atom_Pos)):
            if abs(linalg.norm(Atom_Pos[i]-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1
            if abs(linalg.norm((Atom_Pos[i]+a_1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_1)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm((Atom_Pos[i]+a_2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_2)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_2)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_2)))
    return H_1st


#Plot Eigvals for Panel (c)
#K-Point
K = (2/3)*(b1 + (b2/2))
#M-Point
M = (1/2)*(b1 + b2)

#k-mesh
k_slice = 60 #180 is used for the paper
contour_interval = 0.05 
contour_levels = arange(0, 10, contour_interval)
kx_array = zeros((k_slice, k_slice))
ky_array = zeros((k_slice, k_slice))
e1_array = zeros((k_slice, k_slice))
kx_mesh = linspace(b2[0], b1[0], k_slice, endpoint=False)
ky_mesh = linspace(-1.5*b1[1], 1.5*b2[1], k_slice, endpoint=False)
X, Y = meshgrid(kx_mesh, ky_mesh)

#Eigenvalues Calc. Function
def EigVal_k_all(k_vec):
    eig_all=linalg.eigh(Ham(k_vec, a1, a2))[0]
    return eig_all

#Eigval Calc in k-mesh
kx_index=0
for kx in kx_mesh:
    ky_index=0
    for ky in ky_mesh:
        k_vec=array([kx, ky])
        eig1 = EigVal_k_all(k_vec)[int(len(Atom_Pos)/2-0)]
        e1_array[ky_index][kx_index] = eig1
        ky_index += 1
    kx_index +=1

#Extract Max Value for Plot
Z = e1_array
vmax = Z.max()

#Plot Figure
plt.figure(figsize=(6, 6), dpi=300, facecolor = "#ffffff")
contours = plt.contour(X, Y, Z, 10, colors='black', levels = contour_levels)
extent=[kx_mesh.min(), kx_mesh.max(), ky_mesh.min(), ky_mesh.max()]
plt.imshow(Z, extent=extent, origin='lower', cmap='jet', vmin = 0.0, vmax=vmax, interpolation="none")

#Colorbar Settings
def modify_cax(ax, cax, orientation='vertical'):
    axp = ax.get_position()
    caxp = cax.ax.get_position()
    if orientation=='vertical':
        cax.ax.set_position([caxp.x0, axp.y0, caxp.x1 - caxp.x0, axp.y1 - axp.y0])
    elif orientation=='horizontal':
        cax.ax.set_position([axp.x0, caxp.y0, axp.x1 - axp.x0, caxp.y1 - caxp.y0])
ax = plt.gca()
cax = plt.colorbar(label='Energy in $t$')
modify_cax(ax, cax)

#Brillouin Zone
P1 = (   b1-  b2)*(1/3)
P2 = ( 2*b1+  b2)*(1/3)
P3 = (   b1+2*b2)*(1/3)
P4 = (  -b1+  b2)*(1/3)
P5 = (-2*b1-  b2)*(1/3)
P6 = (  -b1-2*b2)*(1/3)
plt.plot([P1[0], P2[0]], [P1[1], P2[1]], color="red", ls = "-")
plt.plot([P2[0], P3[0]], [P2[1], P3[1]], color="red", ls = "-")
plt.plot([P3[0], P4[0]], [P3[1], P4[1]], color="red", ls = "-")
plt.plot([P4[0], P5[0]], [P4[1], P5[1]], color="red", ls = "-")
plt.plot([P5[0], P6[0]], [P5[1], P6[1]], color="red", ls = "-")
plt.plot([P6[0], P1[0]], [P6[1], P1[1]], color="red", ls = "-")

#Gamma-, K- and M-point
plt.text(0, -0.06, r"$\mathrm{\Gamma}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.text(K[0], K[1]*1.05, "$\mathrm{K}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.text(M[0], M[1]*1.05, "$\mathrm{M}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.quiver(0, 0, K[0], K[1], angles='xy', scale_units='xy', width = 0.0105, lw = 1.0, ec ='w', color = "k", scale=1, zorder = 99)
plt.quiver(K[0], K[1], M[0]-K[0], M[1]-K[1], angles='xy', scale_units='xy', width = 0.0105, lw = 1.0, ec ='w', color = "k", scale=1, zorder = 99)
plt.quiver(M[0], M[1], -M[0], -M[1], angles='xy', scale_units='xy', width = 0.0105, lw = 1.0, ec ='w', color = "k", scale=1, zorder = 99)

#kx, ky Ticks and Label Settings 
plt.xticks([0], size = "x-large")
plt.yticks([0], size = "x-large")
plt.xlabel("$k_x$", size = "x-large")
plt.ylabel("$k_y$", size = "x-large")

#Show Figure
plt.show()


#Plot Eigvals for Panel (d)
e1_array_eff = zeros((k_slice, k_slice)) 

#2-Site Model
tau1 = 6/sqrt(4843)
tau2 = tau1
tau3 = tau1

#Hamiltonian for 2-Site Model
def Ham_2atom(k_vec, a_1, a_2, tau1, tau2, tau3):
    H_0      = zeros((2, 2), dtype=complex64)
    H_1      = zeros((2, 2), dtype=complex64)
    H_2      = zeros((2, 2), dtype=complex64)
    H_0[0][1]= tau1
    H_0[1][0]= tau1
    H_1[0][1]= tau2*exp(-1.0j*(k_vec.dot(a_1)))
    H_1[1][0]= tau2*exp( 1.0j*(k_vec.dot(a_1)))
    H_2[0][1]= tau3*exp(-1.0j*(k_vec.dot(a_2)))
    H_2[1][0]= tau3*exp( 1.0j*(k_vec.dot(a_2)))
    H        = H_0 + H_1 + H_2
    return H 

#Eigenvalues Calc. Function for 2-Site Model
def EigVal_k_2atom(k_vec, a1, a2, tau1, tau2, tau3):
    eig_n=linalg.eigh(Ham_2atom(k_vec, a1, a2, tau1, tau2, tau3))[0][1]
    return eig_n

#Eigval Calc in k-mesh for 2-Site Model            
kx_index=0
for kx in kx_mesh:
    ky_index=0
    for ky in ky_mesh:
        k_vec=array([kx, ky])
        eig1_eff = EigVal_k_2atom(k_vec, a1, a2, tau1, tau2, tau3)
        e1_array_eff[ky_index][kx_index] = eig1_eff
        ky_index += 1
    kx_index +=1
Zeff = e1_array_eff

#Plot Figure
plt.figure(figsize=(6, 6), dpi=300, facecolor = "#ffffff")
contours = plt.contour(X, Y, Zeff, 10, colors='black',  levels = contour_levels)
extent=[kx_mesh.min(), kx_mesh.max(), ky_mesh.min(), ky_mesh.max()]
plt.imshow(Zeff, extent=extent, origin='lower', cmap='jet', vmin = 0.0, vmax=vmax, interpolation="none")

#Colorbar Settings
ax = plt.gca()
cax = plt.colorbar(label='Energy in $t$')
modify_cax(ax, cax)

#Brillouin Zone
plt.plot([P1[0], P2[0]], [P1[1], P2[1]], color="red", ls = "-")
plt.plot([P2[0], P3[0]], [P2[1], P3[1]], color="red", ls = "-")
plt.plot([P3[0], P4[0]], [P3[1], P4[1]], color="red", ls = "-")
plt.plot([P4[0], P5[0]], [P4[1], P5[1]], color="red", ls = "-")
plt.plot([P5[0], P6[0]], [P5[1], P6[1]], color="red", ls = "-")
plt.plot([P6[0], P1[0]], [P6[1], P1[1]], color="red", ls = "-")

#Gamma-, K- and M-point
plt.text(0, -0.06, r"$\mathrm{\Gamma}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.text(K[0], K[1]*1.05, "$\mathrm{K}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.text(M[0], M[1]*1.05, "$\mathrm{M}$", size = "x-large", va = 'bottom', backgroundcolor = 'white')
plt.quiver(0, 0, K[0], K[1], angles='xy', scale_units='xy', width = 0.0105, lw = 1.2, ec ='k', color = "r", ls='--', scale=1, zorder = 99)
plt.quiver(K[0], K[1], M[0]-K[0], M[1]-K[1], width = 0.0105, lw = 1.2, ec ='k', color = "r", ls='--', angles='xy', scale_units='xy', scale=1, zorder = 99)
plt.quiver(M[0], M[1], -M[0], -M[1], width = 0.0105, lw = 1.2, ec ='k', color = "r", ls='--', angles='xy', scale_units='xy', scale=1, zorder = 99)

#kx, ky Ticks and Label Settings 
plt.xticks([0], size = "x-large")
plt.yticks([0], size = "x-large")
plt.xlabel("$k_x$", size = "x-large")
plt.ylabel("$k_y$", size = "x-large")

#Show Figure
plt.show()


#Band Plot for Panel (e)
band_slice = 60

#Band Calc.
Band_K_points = []
#G→K
for i in range(int(2*band_slice)):
    Band_K_points.append(i*K/int(2*band_slice))
#K→M
for i in range(int(1*band_slice)):
    Band_K_points.append((int(1*band_slice)-i)*K/int(1*band_slice)+(i)*M/int(1*band_slice))
#M→G
for i in range(int(sqrt(3)*band_slice)+1):
    Band_K_points.append((int(sqrt(3)*band_slice)-i)*M/int(sqrt(3)*band_slice))

#Original Model Band
EigenEnergy_Band_k = []
for k in range(len(Band_K_points)):    
    EigenEnergy_Band_k.append(EigVal_k_all(Band_K_points[k]))
EigenEnergy_Band = array(EigenEnergy_Band_k).T

#2-site Model Band
EigenEnergy_Eff_k = []
for k in range(len(Band_K_points)):
    EigenEnergy_Eff_k.append(EigVal_k_2atom(Band_K_points[k], a1, a2, tau1, tau2, tau3))
EigenEnergy_Eff = array(EigenEnergy_Eff_k).T

#Plot Figure
plt.figure(figsize=(6,4), dpi=300)
plt.ylabel("Energy in $t$")
plt.xlim(0, len(Band_K_points)-1)
plt.ylim(-1.0, 1.0)
for i in range(len(Atom_Pos)):
    plt.plot(EigenEnergy_Band[i], "k-")
plt.plot( EigenEnergy_Eff, "r--")
plt.plot(-EigenEnergy_Eff, "r--")
plt.xticks([0, int(2*band_slice), int(3*band_slice), len(Band_K_points)-1], [r"$\mathrm{\Gamma}$", "$\mathrm{K}$", "$\mathrm{M}$", r"$\mathrm{\Gamma}$"])
plt.axvline(int(2*band_slice), c='k', ls='--')
plt.axvline(int(3*band_slice), c='k', ls='--')
plt.grid()

#Show Figure
plt.show()