from scipy import *
from numpy import *
import matplotlib.pyplot as plt
plt.rcParams['mathtext.fontset'] = 'cm'

#2D Lattice Vector, Reciprocal Lattice Vector and Energy-Minimum Point Generation 
def a_b_vec2D(a1, a2, a3):
    b1=((2*pi)*cross(a2, a3)/(dot(a1, cross(a2, a3))))[0:2]
    b2=((2*pi)*cross(a3, a1)/(dot(a2, cross(a3, a1))))[0:2]
    b3=((2*pi)*cross(a1, a2)/(dot(a3, cross(a1, a2))))[0:2]
    a1 = a1[0:2] #2D vec.
    a2 = a2[0:2]
    a3 = a3[0:2]
    k_min = (2/3)*(b1 + (b2/2)) #Energy-Minimum Point
    return a1, a2, a3, b1, b2, b3, k_min

#Hopping Matrix Generation
def Ham(k_vec, a_1, a_2, Atom_Pos):
    H_1st = zeros((len(Atom_Pos), len(Atom_Pos)), dtype=complex128)
    for i in range(len(Atom_Pos)):
        for j in range(len(Atom_Pos)):
            if abs(linalg.norm(Atom_Pos[i]-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  += t1
            if abs(linalg.norm((Atom_Pos[i]+a_1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  += t1*exp(-1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_1)) - C_C_len)  < 1e-6:
                H_1st[i][j]  += t1*exp( 1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm((Atom_Pos[i]+a_2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  += t1*exp(-1.0j*(k_vec.dot(a_2)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_2)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  += t1*exp( 1.0j*(k_vec.dot(a_2)))
    return H_1st

#Parameter Set
t1      = -1.0  #Hopping Parameter
C_C_len =  1.42 #C-C lentgh
dkx = array([1e-6, 0.0]) #dkx for Fermi Velocity nu_x Calc.
dky = array([0.0, 1e-6]) #dky for Fermi Velocity nu_y Calc.


#Fermi Velocity of Graphene
#Atom Pos
Atom_Pos_Gra         = zeros([2, 2])
Atom_Pos_Gra[0]      = array([0, 1])*C_C_len
Atom_Pos_Gra[1]      = array([0, 2])*C_C_len

#Lattice Vector
a1_Gra = array([ sqrt(3)/2, 3/2, 0])*C_C_len
a2_Gra = array([-sqrt(3)/2, 3/2, 0])*C_C_len
a3_Gra = array([0, 0, 1])*C_C_len

#Reciprocal Lattice Vector Calc. (2D)
a1, a2, a3, b1, b2, b3, k_min = a_b_vec2D(a1_Gra, a2_Gra, a3_Gra)

#Fermi Velocity Calc.
FermiV_Gra = linalg.eigh(Ham(k_min+dkx, a1, a2, Atom_Pos_Gra))[0][1] #Engergy at k_min + dkx


#Fermi Velocity of Isotropic PU Arms with L_arm = 1 to 31
#Fermi Velocity of L_arm = 1
#File Read
DataFolder = "./AtomPos_UnitCell"
Atom_Pos_File    = DataFolder+"/AtomPos_Fig4_A1a.xyz" #Atomic Positions
Lattice_Vec_File = DataFolder+"/Lattice_Fig4_A1a.csv" #Unit Cell Lattice Vector

#Atomic Positions Read
Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
Atom_Pos       = zeros((len(Atom_Pos_read), 2))
for i in range(len(Atom_Pos)):
    Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])

#Unit Cell Read
Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
a1 = Lattice_Vec[0]
a2 = Lattice_Vec[1]
a3 = Lattice_Vec[2]

#Reciprocal Lattice Vector Calc. (2D)
a1, a2, a3, b1, b2, b3, k_min = a_b_vec2D(a1, a2, a3)

#Engergy at k_min + dkx
FermiV_Larm1 = linalg.eigh(Ham(k_min+dkx, a1, a2, Atom_Pos))[0][int(len(Atom_Pos)/2+0)]

#nu'/nu for L_arm = 1
nu_ratio_1 = FermiV_Larm1/FermiV_Gra
nu_ratio = [nu_ratio_1]

#L_arm from 2 to 31
for L_arm in range(2, 32, 1):
    #File Read
    DataFolder = "./AtomPos_UnitCell"
    Atom_Pos_File    = DataFolder+"/AtomPos_Fig8a_IsoPU_Larm%s.xyz"%int(L_arm) #Atomic Positions
    Lattice_Vec_File = DataFolder+"/Lattice_Fig8a_IsoPU_Larm%s.csv"%int(L_arm) #Unit Cell Lattice Vector
    
    #Atomic Positions Read
    Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
    Atom_Pos       = zeros((len(Atom_Pos_read), 2))
    for i in range(len(Atom_Pos)):
        Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])
    
    #Unit Cell Read
    Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
    a1 = Lattice_Vec[0]
    a2 = Lattice_Vec[1]
    a3 = Lattice_Vec[2]

    #Reciprocal Lattice Vector Calc. (2D)
    a1, a2, a3, b1, b2, b3, k_min = a_b_vec2D(a1, a2, a3)

    #Engergy at k_min + dkx
    FermiV_L_arm = linalg.eigh(Ham(k_min+dkx, a1, a2, Atom_Pos))[0][int(len(Atom_Pos)/2+0)]

    #nu'/nu
    nu_ratio_L_arm = FermiV_L_arm/FermiV_Gra
    nu_ratio.append(nu_ratio_L_arm)

    
#Fermi Velocity of Uniaxis PU Arms with L_arm = 1 to 31
#nu'_x/nu for L_arm = 1
nu_x_ratio = [nu_ratio_1]
#nu'_y/nu for L_arm = 1
nu_y_ratio = [nu_ratio_1]

#L_arm from 2 to 31
for L_arm in range(2, 32, 1):
    #File Read
    DataFolder = "./AtomPos_UnitCell"
    Atom_Pos_File    = DataFolder+"/AtomPos_Fig8b_UniaxPU_Larm%s.xyz"%int(L_arm) #Atomic Positions
    Lattice_Vec_File = DataFolder+"/Lattice_Fig8b_UniaxPU_Larm%s.csv"%int(L_arm) #Unit Cell Lattice Vector

    #Atomic Positions Read
    Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
    Atom_Pos       = zeros((len(Atom_Pos_read), 2))
    for i in range(len(Atom_Pos)):
        Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])

    #Unit Cell Read
    Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
    a1 = Lattice_Vec[0]
    a2 = Lattice_Vec[1]
    a3 = Lattice_Vec[2]

    #Reciprocal Lattice Vector Calc. (2D)
    a1, a2, a3, b1, b2, b3, k_min = a_b_vec2D(a1, a2, a3)

    #Engergy at k_min + dkx
    FermiVx_L_arm = linalg.eigh(Ham(k_min+dkx, a1, a2, Atom_Pos))[0][int(len(Atom_Pos)/2+0)]
    
    #Engergy at k_min + dky
    FermiVy_L_arm = linalg.eigh(Ham(k_min+dky, a1, a2, Atom_Pos))[0][int(len(Atom_Pos)/2+0)]
    
    #nu'_x/nu
    nu_x_ratio_L_arm = FermiVx_L_arm/FermiV_Gra
    nu_x_ratio.append(nu_x_ratio_L_arm)
    
    #nu'_y/nu
    nu_y_ratio_L_arm = FermiVy_L_arm/FermiV_Gra
    nu_y_ratio.append(nu_y_ratio_L_arm)
    

#Data Plot 
x  = arange(1, 32, 1) 
y1 = nu_ratio   
y2 = nu_x_ratio  
y3 = nu_y_ratio  
plt.figure(figsize=(5.0, 4.6), dpi = 400)
plt.plot(x, y1,  "go", label=r'$\nu^{\prime~}/\nu$ (TB calc.)', zorder = 101)
plt.plot(x, y2,  'bs', label=r'$\nu^{\prime\prime}_{x}/\nu$ (TB calc.)', zorder = 100)
plt.plot(x, y3,  'r^', label=r'$\nu^{\prime\prime}_{y}/\nu$ (TB calc.)', zorder = 99)

#Analytical Solutions
xx = arange(1, 31, 0.001)
fit_curve1 = (12 * xx -4) / (24 * xx -13)
fit_curve2 = (8 *  1) / (8 * xx + 3)
fit_curve3 = (8 * xx) / (8 * xx + 3)
plt.plot(xx, fit_curve1, 'g-', label=r'$(12L_{\mathrm{arm}}-4)/(24L_{\mathrm{arm}}-13)$', zorder = 11)
plt.plot(xx, fit_curve2, 'b-', label=r'$8                /(8L_{\mathrm{arm}}+3)$', zorder = 10)
plt.plot(xx, fit_curve3, 'r-', label=r'$8L_{\mathrm{arm}}/(8L_{\mathrm{arm}}+3)$', zorder = 9)

#Plot Settings
plt.axhline(y=1, color='red', linestyle='--')
plt.axhline(y=0, color='blue', linestyle='--')
plt.axhline(y=0.5, color='green', linestyle='--')
plt.xlim(0.8, 31)
plt.xticks(arange(1, 32, 5))
plt.xlabel('$L_{\mathrm{arm}}$')
plt.ylim(-0.02, 1.02)
plt.yticks(arange(0.0, 1.1, 0.1))
plt.ylabel('Fermi velocity ratio')
plt.legend(loc='upper right', bbox_to_anchor=(1, 0.95))
plt.grid(which='both', linestyle='-', color='gray')

#Plot Show
plt.show()