from scipy import *
from numpy import *
import matplotlib.pyplot as plt

#File Read
DataFolder = "./AtomPos_UnitCell"
Atom_Pos_File    = DataFolder+"/AtomPos_Fig5_A1cd.xyz" #Atomic Positions
Lattice_Vec_File = DataFolder+"/Lattice_Fig5_A1cd.csv" #Unit Cell Lattice Vector

#Parameter Set
t1      = -1.0  #Hopping Parameter
C_C_len =  1.42 #C-C lentgh

#Atomic Positions Read
Atom_Pos_read  = genfromtxt(Atom_Pos_File, skip_header = 2, delimiter=" ")
Atom_Pos       = zeros((len(Atom_Pos_read), 2))
for i in range(len(Atom_Pos)):
    Atom_Pos[i] = array([Atom_Pos_read[i][1], Atom_Pos_read[i][2]])

#Unit Cell Read
Lattice_Vec = loadtxt(Lattice_Vec_File, delimiter=",")
a1 = Lattice_Vec[0]
a2 = Lattice_Vec[1]
a3 = Lattice_Vec[2]

#Reciprocal lattice vector calc. (2D)
b1=((2*pi)*cross(a2, a3)/(dot(a1, cross(a2, a3))))[0:2]
b2=((2*pi)*cross(a3, a1)/(dot(a2, cross(a3, a1))))[0:2]
b3=((2*pi)*cross(a1, a2)/(dot(a3, cross(a1, a2))))[0:2]
a1 = Lattice_Vec[0][0:2] #2D vec.
a2 = Lattice_Vec[1][0:2]
a3 = Lattice_Vec[2][0:2]

#Hopping Matrix Generation
def Ham(k_vec, a_1, a_2):
    H_1st = zeros((len(Atom_Pos), len(Atom_Pos)), dtype=complex64)
    for i in range(len(Atom_Pos)):
        for j in range(len(Atom_Pos)):
            if abs(linalg.norm(Atom_Pos[i]-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1
            if abs(linalg.norm((Atom_Pos[i]+a_1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_1)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_1)))
            if abs(linalg.norm((Atom_Pos[i]+a_2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp(-1.0j*(k_vec.dot(a_2)))
            if abs(linalg.norm(Atom_Pos[i]-(Atom_Pos[j]+a_2)) - C_C_len)  < 1e-6: 
                H_1st[i][j]  = t1*exp( 1.0j*(k_vec.dot(a_2)))
    return H_1st

#K-Point
K = (2/3)*(b1 + (b2/2))

#Hamiltonian Matrix at K-point
Ham_K = Ham(K, a1, a2)

#Eigenvectors Plot
wavsize = 115.0 #Size of Circle
wavalph = 0.5   #Alpha of Circle
plt.rcParams['mathtext.fontset'] = 'cm'

#Eigvecs Calc.
def EigVec_kxky_E0(k_vec):
    eig_vec_0=linalg.eigh(Ham(k_vec, a1, a2))[1].T[int(len(Atom_Pos)/2-0)]
    eig_vec_1=linalg.eigh(Ham(k_vec, a1, a2))[1].T[int(len(Atom_Pos)/2-1)]
    return eig_vec_0, eig_vec_1

#Eigvecs Split on A/B sites
d_k         = array([0.0, 0.000001])#for avoiding numerical degeneracy
E0_EigVec_0 =  (EigVec_kxky_E0(K+d_k)[1] - EigVec_kxky_E0(K+d_k)[0])/sqrt(2)*exp(1.0j*pi/2)
E0_EigVec_1 =  (EigVec_kxky_E0(K+d_k)[0] + EigVec_kxky_E0(K+d_k)[1])/sqrt(2)*exp(2.0j*pi/2)

#Transform Complex to Cmap and Circle Size
def Complex2RGBhsv(C):
    AbsVal = abs(C)
    if angle(C) >= 0.0:
        Phase  = angle(C)
    elif angle(C) < 0.0:
        Phase  = angle(C) + 2.0*pi
    DegPh  = degrees(Phase)
    cmap_int = round((DegPh/360)*256)
    cmap = plt.get_cmap("hsv")
    RGB_params = cmap(cmap_int) 
    return AbsVal, Phase, DegPh, (RGB_params[0], RGB_params[1], RGB_params[2])

#Plot Eigvec on A-sites
plt.figure(figsize = (10, 10), dpi = 300)
plt.xlim(-max(a1[0], a1[1], a2[0], a2[1])*1.1, max(a1[0], a1[1], a2[0], a2[1])*1.1)
plt.ylim(-max(a1[0], a1[1], a2[0], a2[1])*0.1, max(a1[0], a1[1], a2[0], a2[1])*2.1)

#Unit Cell Plot
plt.quiver(0, 0, a1[0], a1[1], color = "black",angles = 'xy', scale_units = 'xy', scale = 1)
plt.quiver(0, 0, a2[0], a2[1], color = "black",angles = 'xy', scale_units = 'xy', scale = 1)
plt.plot([a1[0], a1[0]+a2[0]], [a1[1], a1[1]+a2[1]], color="black", ls = "--")
plt.plot([a2[0], a1[0]+a2[0]], [a2[1], a1[1]+a2[1]], color="black", ls = "--")

#Eigvec Plot
for i in range(len(Atom_Pos)):
    RGB = Complex2RGBhsv(E0_EigVec_0[i])
    plt.plot(Atom_Pos[i][0], Atom_Pos[i][1], marker = "o", c =RGB[3], ms = RGB[0]*wavsize, alpha = wavalph)

#Plot C-C bonds
for i in range(len(Atom_Pos)):
    for j in range(i+1, len(Atom_Pos)):
        if abs(Ham_K[i][j] - t1) < 1e-6: 
            plt.plot([Atom_Pos[i][0], Atom_Pos[j][0]], [Atom_Pos[i][1], Atom_Pos[j][1]], "k-", lw=abs(t1)) 
for i in range(len(Atom_Pos)):
    for j in range(len(Atom_Pos)):
        if abs(Ham_K[i][j] - t1*exp(-1.0j*(K.dot(a1)))) < 1e-6 and abs(linalg.norm((Atom_Pos[i]+a1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
            plt.plot([Atom_Pos[i][0]+a1[0], Atom_Pos[j][0]], [Atom_Pos[i][1]+a1[1], Atom_Pos[j][1]], "k-", lw=abs(t1)) 
        if abs(Ham_K[i][j] - t1*exp(-1.0j*(K.dot(a2)))) < 1e-6 and abs(linalg.norm((Atom_Pos[i]+a2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
            plt.plot([Atom_Pos[i][0]+a2[0], Atom_Pos[j][0]], [Atom_Pos[i][1]+a2[1], Atom_Pos[j][1]], "k-", lw=abs(t1))       
#Show Plot
plt.gca().axis("off")
plt.show()

#Plot Eigvec on B-sites
plt.figure(figsize = (10, 10), dpi = 300)
plt.xlim(-max(a1[0], a1[1], a2[0], a2[1])*1.1, max(a1[0], a1[1], a2[0], a2[1])*1.1)
plt.ylim(-max(a1[0], a1[1], a2[0], a2[1])*0.1, max(a1[0], a1[1], a2[0], a2[1])*2.1)

#Unit Cell Plot
plt.quiver(0, 0, a1[0], a1[1], color = "black",angles = 'xy', scale_units = 'xy', scale = 1)
plt.quiver(0, 0, a2[0], a2[1], color = "black",angles = 'xy', scale_units = 'xy', scale = 1)
plt.plot([a1[0], a1[0]+a2[0]], [a1[1], a1[1]+a2[1]], color="black", ls = "--")
plt.plot([a2[0], a1[0]+a2[0]], [a2[1], a1[1]+a2[1]], color="black", ls = "--")

#Eigvec Plot
for i in range(len(Atom_Pos)):
    RGB = Complex2RGBhsv(E0_EigVec_1[i])
    plt.plot(Atom_Pos[i][0], Atom_Pos[i][1], marker = "o", c =RGB[3], ms = RGB[0]*wavsize, alpha = wavalph)

#Plot C-C bonds
for i in range(len(Atom_Pos)):
    for j in range(i+1, len(Atom_Pos)):
        if abs(Ham_K[i][j] - t1) < 1e-6: 
            plt.plot([Atom_Pos[i][0], Atom_Pos[j][0]], [Atom_Pos[i][1], Atom_Pos[j][1]], "k-", lw=abs(t1)) 
for i in range(len(Atom_Pos)):
    for j in range(len(Atom_Pos)):
        if abs(Ham_K[i][j] - t1*exp(-1.0j*(K.dot(a1)))) < 1e-6 and abs(linalg.norm((Atom_Pos[i]+a1)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
            plt.plot([Atom_Pos[i][0]+a1[0], Atom_Pos[j][0]], [Atom_Pos[i][1]+a1[1], Atom_Pos[j][1]], "k-", lw=abs(t1)) 
        if abs(Ham_K[i][j] - t1*exp(-1.0j*(K.dot(a2)))) < 1e-6 and abs(linalg.norm((Atom_Pos[i]+a2)-Atom_Pos[j]) - C_C_len)  < 1e-6: 
            plt.plot([Atom_Pos[i][0]+a2[0], Atom_Pos[j][0]], [Atom_Pos[i][1]+a2[1], Atom_Pos[j][1]], "k-", lw=abs(t1))       

#Show Plot
plt.gca().axis("off")
plt.show()